#!/usr/bin/env python
#
# Copyright (C) 2011 Greg Auger <gregory.auger@gmail.com>
#
# This file is part of FBuntu.
#
# FBuntu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FBuntu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Fbuntu.  If not, see <http://www.gnu.org/licenses/>.

import auth, facebook
import notificationChecker, unifiedInboxChecker, legacyInboxChecker, graphNotificationChecker
import logging
import indicate, sys, webbrowser

class fbuntu:
    def __init__(self):
        configfile = sys.path[0] + "/debug.log"
        logging.basicConfig(filename=configfile,level=logging.DEBUG,format='%(asctime)s %(filename)s:%(lineno)d %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')
        self.logger = logging.getLogger("Fbuntu")

        self.oauth_access_token = auth.facebookAuthentication().oauth_access_token
        self.graph = facebook.GraphAPI(self.oauth_access_token)

        self.userIDDictionary = {}
        self.checkCount = 0

        self.checkers = []
        self.checkers.append(graphNotificationChecker.graphNotificationChecker(self))
        #self.checkers.append(unifiedInboxChecker.unifiedInboxChecker(self))        
        #self.checkers.append(legacyInboxChecker.legacyInboxChecker(self)) # this is the line to enable if you would like to try getting notifications for inbox messages
        #self.checkers.append(pokesChecker()) # not yet implemented, so don't try enabling it!

        import pynotify
        pynotify.init("fbuntu")

        self.mm = indicate.indicate_server_ref_default()
        self.mm.connect("server-display", self.mmClicked)
        self.mm.set_type("message.im")
        desktop_file = sys.path[0] + "/FBuntu.desktop"
        self.mm.set_desktop_file(desktop_file)
        self.mm.show()

    def update(self, returnValue):
        self.checkCount += 1
        self.logger.info("Update #%d starting" % (self.checkCount))

        for checker in self.checkers:
            checker.recheck()

        self.logger.info("Ended update #%d\n" % (self.checkCount))

        return returnValue # neccessary for gobject.timeout_add_seconds() to continue

    def mmClicked(self, indicator, time):
        webbrowser.open("http://www.facebook.com")

    def pictureFromId(self, userID):
        import os, urllib, urllib2

        if userID == None:
            return None

        try:
            os.makedirs(os.getenv('HOME') + "/.fbuntu/icons")
        except OSError:
            pass

        url = "https://graph.facebook.com/" + userID + "/picture?type=square"
        local_file = os.getenv('HOME') + "/.fbuntu/icons/" + userID + ".jpg"

        if not os.path.exists(local_file):
            success = 0
            self.logger.info("No picture for user %s, downloading new copy" % (userID))            
            while success == 0:
                try:
                    urllib.urlretrieve(url, local_file)
                except urllib2.HTTPError, e:
                    self.logger.warning("The server couldn't fulfill the request. Error code: %s" % (e.code))
                    time.sleep(15)
                except urllib2.URLError, e:
                    self.logger.warning("We failed to reach a server. Reason: %s" % (e.reason))
                    time.sleep(15)
                except IOError, e:
                    self.logger.warning("IOError. More info: %s" % (e))
                    time.sleep(15)
                except Exception, e:
                    self.logger.warning("Exception. More info: %s" % (e))
                    time.sleep(15)
                else:
                    success = 1

        return local_file

    def nameFromId(self, userID):
        userID = str(userID)
        try:
            name = self.userIDDictionary[userID]
        except KeyError:
            try:
                user = self.graph.get_object(userID)
            except:
                self.logger.warning("User %s not in dictionary and FB can't find them." % (userID))
                return str(userID)
            else:
                self.userIDDictionary[userID] = user['name']
                self.logger.debug("User %s not in dictionary but FB says they're called %s." % (userID,user['name']))
                return user['name']
        else:
            self.logger.debug("User %s was in dictionary as %s." % (userID,name))
            return name



if __name__ == "__main__":

    import sys, traceback, os
    
    # Shorten log
    os.system('tail -n 500 debug.log > debug.short')
    os.system('mv debug.short debug.log ')
    
    logger = logging.getLogger("Fbuntu2")
    
    def custom_exceptionHook(type, value, tb):
        # Print to running terminal
        traceback.print_exception(type, value, tb)

        # Put exception in debug file
        exception = traceback.format_exception(type, value, tb)
        string = ""
        for line in exception:
            string = string + line
        logger.error(string)

        # Notify user using zenity
        os.system('zenity --info --text="My apologies.\n\nFBuntu has encountered an error.\n\nPlease see the logs for full details.\n\nFBuntu has quit."' % (value))
        
    sys.excepthook = custom_exceptionHook

    import gtk, gobject

    a = fbuntu()
    timer = gobject.timeout_add_seconds(1, a.update, 0)
    timer = gobject.timeout_add_seconds(20, a.update, 1)
    gtk.main()

