#!/usr/bin/env python
#
# Copyright (C) 2011 Greg Auger <gregory.auger@gmail.com>
#
# This file is part of FBuntu.
#
# FBuntu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FBuntu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Fbuntu.  If not, see <http://www.gnu.org/licenses/>.

import logging, urllib, urllib2, json, time

class graphNotificationChecker:
    def __init__(self, parent):
        self.logger = logging.getLogger(__name__)
        self.parent = parent
        self.oauth_access_token = parent.oauth_access_token
        self.notificationArray = []
        self.graph = parent.graph
        self.user = self.graph.get_object("me")

    def recheck(self):
        self.logger.info("Checking Facebook")
        self.retrieve()
        self.processResponse()

    def retrieve(self):
        self.response = self.graph.get_connections(self.user["id"], "notifications")['data']
        self.logger.debug(self.response)

    def processResponse(self):

        # checks each message system knows, to see if still considered unread by fb
        for notificationInArray in self.notificationArray:
            snapped = False
            for response in self.response:
                if response['id'] == notificationInArray.response['id']:
                    snapped = True # message still unread
                    self.logger.debug("%s is still unread" % (notificationInArray.response['id']))
            if snapped == False: # message no longer unread
                self.logger.debug("%s is no longer unread" % (notificationInArray.response['id']))
                notificationInArray.indicator.hide()
                self.notificationArray.remove(notificationInArray)
        # check each message in response against array
        for response in self.response:
            notificationAlreadyKnown = False

            for arrayNotification in self.notificationArray:
                if response['id'] == arrayNotification.response['id']:
                    index = self.notificationArray.index(arrayNotification)
                    self.logger.debug("Already know about notification %s - in notificationArray at index %d" % (response['id'], index))
                    notificationAlreadyKnown = True
                    self.notificationArray[index].update(response)

            if notificationAlreadyKnown == False:
                self.logger.debug("Notification %s not previously known about" % (response['id']))
                self.notificationArray.append(notification(self, response))

import indicate, pynotify

class notification():
    def __init__(self, parent, notification):
        self.parent = parent
        self.response = notification

        if not 'message' in self.response:
            self.response['message'] = ""
        if not 'link' in self.response:
            self.response['link'] = "http://facebook.com"

        self.indicator = indicate.Indicator()
        self.indicator.set_property("subtype", "im")
        self.indicator.connect("user-display", self.mmClicked)
        self.indicator.set_property("draw-attention", "true")
        self.update(notification, True)
        self.indicator.show()

    def update(self, notification, forceUpdate=False):
        if (notification['updated_time'] != self.response['updated_time']) or (forceUpdate == True):        
            self.response = notification
            if not 'message' in self.response:
                self.response['message'] = ""
            if not 'link' in self.response:
                self.response['link'] = "http://facebook.com"
            self.image = self.parent.parent.pictureFromId(self.response['from']['id'])
            self.mmUpdate()
            self.notification = pynotify.Notification(self.response['title'], self.response['message'], self.image)
            self.notification.show()

    def mmUpdate(self):
        mmText = self.response['title']
        self.indicator.set_property("sender", mmText)
        import dateutil.parser
        dt = dateutil.parser.parse(self.response['updated_time'])
        self.indicator.set_property_time("time", int(dt.strftime("%s")))
        self.website = self.response['link']

        if not self.image == None:
            import gtk
            #convert the imge to a pixbuf
            pixbuf=gtk.gdk.pixbuf_new_from_file(self.image)
            self.indicator.set_property_icon("icon", pixbuf)

    def mmClicked(self, a, b):
        # a is the same as self.indicator
        # b is an integer, likely time of sort sort
        import webbrowser
        webbrowser.open(self.website)
        self.indicator.hide()
        self.parent.logger.info("Deleting notification %s because clicked on.\n" % (self.response['id']))
        self.parent.notificationArray.remove(self) # remove ourself from our parent's array
