#!/usr/bin/env python
#
# Copyright (C) 2011 Greg Auger <gregory.auger@gmail.com>
#
# This file is part of FBuntu.
#
# FBuntu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FBuntu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Fbuntu.  If not, see <http://www.gnu.org/licenses/>.

import logging, urllib, urllib2, json, time

class legacyInboxChecker:
    def __init__(self, parent):
        self.logger = logging.getLogger(__name__)
        self.parent = parent
        self.oauth_access_token = parent.oauth_access_token
        self.messageArray = []

    def recheck(self):
        self.logger.info("Checking Facebook")
        self.retrieve()
        self.processResponse()

    def retrieve(self):
        query = "SELECT subject,snippet,snippet_author,updated_time,thread_id FROM thread WHERE folder_id = 0 AND unread != 0"
        query = urllib.quote(query)
        url = "https://api.facebook.com/method/fql.query?" + "format=json" + "&access_token=" + self.oauth_access_token + "&query=" + query

        success = 0
        while success == 0:
            try:
                self.response = json.loads(urllib2.urlopen(url,timeout=15).read())
            except urllib2.HTTPError, e:
                self.logger.warning("The server couldn't fulfill the request. Error code: %s" % (e.code))
                time.sleep(15)
            except urllib2.URLError, e:
                self.logger.warning("We failed to reach a server. Reason: %s" % (e.reason))
                time.sleep(15)
            except IOError, e:
                self.logger.warning("IOError. More info: %s" % (e))
                time.sleep(15)
            except Exception, e:
                self.logger.warning("Exception. More info: %s" % (e))
                time.sleep(15)                
            else:
                success = 1

        try:
            self.response['error_code']
        except:
            pass # good news, we haven't found an error code
        else:
            self.logger.error("Error %s %s" % (self.response['error_code'], self.response['error_msg']))
            import sys
            sys.exit()

        self.logger.debug(self.response)

    def processResponse(self):

        # checks each message system knows, to see if still considered unread by fb
        for inboxMessageInArray in self.messageArray:
            snapped = False
            for response in self.response:
                if response['thread_id'] == inboxMessageInArray.response['thread_id']:
                    snapped = True # message still unread
                    self.logger.debug("%s is still unread" % (inboxMessageInArray.response['thread_id']))
            if snapped == False: # message no longer unread
                self.logger.debug("%s is no longer unread" % (inboxMessageInArray.response['thread_id']))
                inboxMessageInArray.indicator.hide()
                self.messageArray.remove(inboxMessageInArray)

        # check each message in response against array
        for response in self.response:

            # skip thread if in blacklist (see https://answers.launchpad.net/fbuntu/+faq/1772)
            # this is just an example of the format of a blacklist
            blacklist = ['2040095356132', '1981718745732', 't_hQvKmvIIpOwByHHhUniOKA', '2220209462240', '2158909489034', '2380320350111']
            if response['thread_id'] in blacklist:
                self.logger.debug("Skipping thread %s because it is blacklisted, see https://answers.launchpad.net/fbuntu/+faq/1772" % (response['thread_id']))
                continue

            threadAlreadyKnown = False

            for arrayInboxMessage in self.messageArray:
                if response['thread_id'] == arrayInboxMessage.response['thread_id']:
                    index = self.messageArray.index(arrayInboxMessage)
                    self.logger.debug("Already know about thread %s - in messageArray at index %d" % (response['thread_id'], index))
                    threadAlreadyKnown = True
                    self.messageArray[index].update(response)

            if threadAlreadyKnown == False:
                self.logger.debug("Thread %s not previously known about" % (response['thread_id']))
                self.messageArray.append(inboxMessage(self,response))

import indicate, pynotify

class inboxMessage():
    def __init__(self, parent, message):
        self.parent = parent

        self.response = message

        self.indicator = indicate.Indicator()
        self.indicator.set_property("subtype", "im")
        self.indicator.connect("user-display", self.mmClicked)
        self.indicator.set_property("draw-attention", "true")
        self.update(message, True)
        self.indicator.show()

    def update(self, message, forceUpdate=False):
        if (message != self.response) or (forceUpdate == True):
            self.response = message
            self.mmUpdate()
            self.notification = pynotify.Notification(self.parent.parent.nameFromId(self.response['snippet_author']), self.response['snippet'], "If you see this, let me know...")
            self.notification.show()

    def mmUpdate(self):
        mmText = self.parent.parent.nameFromId(self.response['snippet_author']) + " sent you a message"
        self.indicator.set_property("sender", mmText)
        self.indicator.set_property_time("time", int(self.response['updated_time']))
        self.website = "http://facebook.com"

    def mmClicked(self, a, b):
        # a is the same as self.indicator
        # b is an integer, likely time of sort sort
        import webbrowser
        webbrowser.open(self.website)
        self.indicator.hide()
        self.parent.logger.info("Deleting thread %s because clicked on.\n" % (self.response['thread_id']))
        self.parent.messageArray.remove(self) # remove ourself from our parent's array
