#!/usr/bin/env python
#
# Copyright (C) 2011 Greg Auger <gregory.auger@gmail.com>
#
# This file is part of FBuntu.
#
# FBuntu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FBuntu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Fbuntu.  If not, see <http://www.gnu.org/licenses/>.

import logging, urllib, urllib2, json, time

class notificationChecker:
    def __init__(self, parent):
        self.logger = logging.getLogger(__name__)
        self.parent = parent
        self.oauth_access_token = parent.oauth_access_token
        self.notificationArray = []

    def recheck(self):
        self.logger.info("Checking Facebook")
        self.retrieve()
        self.processResponse()

    def retrieve(self):
        query = "SELECT notification_id,sender_id,created_time,title_text,body_text,href FROM notification WHERE is_unread = 1 AND is_hidden = 0 AND recipient_id = me()"
        query = urllib.quote(query)
        url = "https://api.facebook.com/method/fql.query?" + "format=json" + "&access_token=" + self.oauth_access_token + "&query=" + query

        success = 0
        while success == 0:
            try:
                self.response = json.loads(urllib2.urlopen(url,timeout=15).read())
            except urllib2.HTTPError, e:
                self.logger.warning("The server couldn't fulfill the request. Error code: %s" % (e.code))
                time.sleep(15)
            except urllib2.URLError, e:
                self.logger.warning("We failed to reach a server. Reason: %s" % (e.reason))
                time.sleep(15)
            else:
                success = 1

        try:
            self.response['error_code']
        except:
            pass # good news, we haven't found an error code
        else:
            self.logger.error("Error %s %s" % (self.response['error_code'], self.response['error_msg']))
            import sys
            sys.exit()

        self.logger.debug(self.response)

    def processResponse(self):

        # checks each message system knows, to see if still considered unread by fb
        for notificationInArray in self.notificationArray:
            snapped = False
            for response in self.response:
                if response['notification_id'] == notificationInArray.response['notification_id']:
                    snapped = True # message still unread
                    self.logger.debug("%s is still unread" % (notificationInArray.response['notification_id']))
            if snapped == False: # message no longer unread
                self.logger.debug("%s is no longer unread" % (notificationInArray.response['notification_id']))
                notificationInArray.indicator.hide()
                self.notificationArray.remove(notificationInArray)

        # check each message in response against array
        for response in self.response:

            notificationAlreadyKnown = False

            for arrayNotification in self.notificationArray:
                if response['notification_id'] == arrayNotification.response['notification_id']:
                    index = self.notificationArray.index(arrayNotification)
                    self.logger.debug("Already know about notification %s - in notificationArray at index %d" % (response['notification_id'], index))
                    notificationAlreadyKnown = True
                    self.notificationArray[index].update(response)

            if notificationAlreadyKnown == False:
                self.logger.debug("Notification %s not previously known about" % (response['notification_id']))
                self.notificationArray.append(notification(self,response))

import indicate, pynotify

class notification():
    def __init__(self, parent, notification):
        self.parent = parent

        self.response = notification

        self.indicator = indicate.Indicator()
        self.indicator.set_property("subtype", "im")
        self.indicator.connect("user-display", self.mmClicked)
        self.indicator.set_property("draw-attention", "true")
        self.update(notification, True)
        self.indicator.show()

    def update(self, notification, forceUpdate=False):
        if (notification != self.response) or (forceUpdate == True):
            self.response = notification
            self.mmUpdate()
            self.notification = pynotify.Notification(self.response['title_text'], self.response['body_text'], "If you see this, let me know...")
            self.notification.show()

    def mmUpdate(self):
        mmText = self.response['title_text']
        self.indicator.set_property("sender", mmText)
        self.indicator.set_property_time("time", int(self.response['created_time']))
        if self.response['href'] != None:
            self.website = self.response['href']
        else:
            self.website = "https://www.facebook.com"

    def mmClicked(self, a, b):
        # a is the same as self.indicator
        # b is an integer, likely time of sort sort
        import webbrowser
        webbrowser.open(self.website)
        self.indicator.hide()
        self.parent.logger.info("Deleting notification %s because clicked on.\n" % (self.response['notification_id']))
        self.parent.notificationArray.remove(self) # remove ourself from our parent's array
